/* ========================================================================= *
 * hashtable
 *
 * hashtables are associative containers that store elements as key-value
 * pairs.  The hashtable here associates a double value to character string keys.
 * ========================================================================= */

#ifndef _HASHTABLE_H_
#define _HASHTABLE_H_

#include <stddef.h>

/* Opaque Structure */
typedef struct hashtable_t hashtable;

/* ------------------------------------------------------------------------- *
 * Create a new empty hashtable object of initial capacity $capacity$.
 *
 * The hashtable object must later be deleted by calling freehashtable().
 *
 * PARAMETERS
 * capacity         Initial container size.
 *
 * RETURN
 * hashtable    A new pointer to a hashtable object.
 * NULL         If an error has occured. The function does not produces any
 *              error message.
 *
 * ------------------------------------------------------------------------- */
hashtable *createhashtable(size_t capacity);

/* ------------------------------------------------------------------------- *
 * Free the allocated memory of the hashtable object $hashTable$ but not its
 * strings elements.
 *
 * PARAMETERS
 * hashTable       An hashtable object
 * ------------------------------------------------------------------------- */
void freehashtable(hashtable *hashtable);

/* ------------------------------------------------------------------------- *
 * Insert a new string-double pair into the hashtable object $hashTable$.
 * Because strings in the hashtable are unique, an insertion overrides the
 * the value associated to a string if an equivalent string is already contained in
 * in the hashtable $hashTable$. In such case, the function returns the old
 * value.
 *
 * Note that the function copies addresses of strings, but does not
 * clone their contents.
 *
 * PARAMETERS
 * hashTable       An hashtable object
 * string          An string value
 * value           The value associated to $string$
 *
 * RETURN
 * old value       The value previously associated to $string$
 * 0.0             Otherwise (also on error).
 *                 The function does not produces any error message.
 * ------------------------------------------------------------------------- */
double insertelement(hashtable *hashtable, char *string, double value);

/* ------------------------------------------------------------------------- *
 * Return the number of elements in the hashtable $hashTable$.
 *
 * PARAMETERS
 * hashTable       An hashtable object
 *
 * RETURN
 * size_t          The number of elements
 * ------------------------------------------------------------------------- */
size_t getnumelements(const hashtable *hashtable);

/* ------------------------------------------------------------------------- *
 * Get value associated to string $string$ in the hashtable $hashTable$.
 *
 * PARAMETERS
 * hashTable       An hashtable object
 * string          The string key
 *
 * RETURN
 * value           double associated to string (0 if not present)
 * ------------------------------------------------------------------------- */
double getvalue(const hashtable *hashtable, const char *string);

#endif // !_HASHTABLE_H_
